library Clock;

uses
  drawShare,Windows,kol;

type
  TTypeArrows = (arEdge, arRectangle, arTriangle);

  TClockTime = packed record
    Hour, Minute, Second: Word;
  end;

type
  PPointArray = ^TPointArray;
  TPointArray = array [0..60 * 2 - 1] of TSmallPoint;

type
  PPoints = ^TPoints;
  TPoints = array[0..0] of TPoint;

type
  TRGB = record r,g,b,x:byte; end;
  PColor = ^TColor;

const
  ClockData: array[0..60 * 4 - 1] of Byte = (
    $00, $00, $C1, $E0, $44, $03, $EC, $E0, $7F, $06, $6F, $E1,
    $A8, $09, $48, $E2, $B5, $0C, $74, $E3, $9F, $0F, $F0, $E4,
    $5E, $12, $B8, $E6, $E9, $14, $C7, $E8, $39, $17, $17, $EB,
    $48, $19, $A2, $ED, $10, $1B, $60, $F0, $8C, $1C, $4B, $F3,
    $B8, $1D, $58, $F6, $91, $1E, $81, $F9, $14, $1F, $BC, $FC,
    $40, $1F, $00, $00, $14, $1F, $44, $03, $91, $1E, $7F, $06,
    $B8, $1D, $A8, $09, $8C, $1C, $B5, $0C, $10, $1B, $A0, $0F,
    $48, $19, $5E, $12, $39, $17, $E9, $14, $E9, $14, $39, $17,
    $5E, $12, $48, $19, $9F, $0F, $10, $1B, $B5, $0C, $8C, $1C,
    $A8, $09, $B8, $1D, $7F, $06, $91, $1E, $44, $03, $14, $1F,
    $00, $00, $3F, $1F, $BC, $FC, $14, $1F, $81, $F9, $91, $1E,
    $58, $F6, $B8, $1D, $4B, $F3, $8C, $1C, $60, $F0, $10, $1B,
    $A2, $ED, $48, $19, $17, $EB, $39, $17, $C7, $E8, $E9, $14,
    $B8, $E6, $5E, $12, $F0, $E4, $9F, $0F, $74, $E3, $B5, $0C,
    $48, $E2, $A8, $09, $6F, $E1, $7F, $06, $EC, $E0, $44, $03,
    $C1, $E0, $00, $00, $EC, $E0, $BC, $FC, $6F, $E1, $81, $F9,
    $48, $E2, $58, $F6, $74, $E3, $4B, $F3, $F0, $E4, $60, $F0,
    $B8, $E6, $A2, $ED, $C7, $E8, $17, $EB, $17, $EB, $C7, $E8,
    $A2, $ED, $B8, $E6, $61, $F0, $F0, $E4, $4B, $F3, $74, $E3,
    $58, $F6, $48, $E2, $81, $F9, $6F, $E1, $BC, $FC, $EC, $E0);

const
  Depth: array[0..59] of Byte = (
    40, 42, 44, 46, 48, 50, 52, 54, 56, 58,
    60, 58, 56, 54, 52, 50, 48, 46, 44, 42,
    40, 38, 36, 34, 32, 30, 28, 26, 24, 22,
    20, 18, 16, 14, 12, 10,  8,  6,  4,  2,
     0,  2,  4,  6,  8, 10, 12, 14, 16, 18,
    20, 22, 24, 26, 28, 30, 32, 34, 36, 38);

const
  MaxDotWidth   = 25; { maximum Hour-marking dot width  }
  MinDotWidth   = 2;  { minimum Hour-marking dot width  }
  MinDotHeight  = 1;  { minimum Hour-marking dot height }
  MinSizeIcon    = 4;  { minimum IconSize }
  FactorSizeIcon = 200 / 24;

  { conversion factors }
  CirTabScale = 8000; { circle table values scale down value  }
  MmPerDm     = 100;  { millimeters per decimeter             }

  { number of hand positions on... }
  HandPositions = 60;                    { ...entire clock         }
  SideShift     = (HandPositions div 4); { ...90 degrees of clock  }
  TailShift     = (HandPositions div 2); { ...180 degrees of clock }

var   DT:PDrawTools;

type
  TLoc = record
     CircleTab: PPointArray;
     HRes: Integer;    { width of the display (in pixels)                    }
     VRes: Integer;    { height of the display (in raster lines)             }
     AspectH: Longint; { number of pixels per decimeter on the display       }
     AspectV: Longint; { number of raster lines per decimeter on the display }
     DB: PBitmap;
     BackBmp: PBitmap;
     ClockBmp: PBitmap;
     FDispTime: TClockTime;
     FClientRect:Trect;
     FClockRadius: Longint;
     FClockCenter: TPoint;
     PictureBack:HBITMAP;
     PictureClock:HBITMAP;
  end;
  PLoc = ^TLoc;

function GetTime(PRec:PParamRec; ed:pointer):TClockTime;
var   ST:TSystemTime;
begin
   GetLocalTime(ST);
   with ST, Result do begin
      Hour   := (wHour + integer(SearchParam(PRec, 'HourOffset').Value^)) mod 24;;
      Minute := wMinute;
      Second := wSecond;
   end;
end;

procedure DrawAnalogFace(PRec:PParamRec; ed:pointer);
var   Pos, DotHeight, DotWidth: Integer;
      DotCenter: TPoint;
      R: TRect;
      SaveBrush, SavePen: TColor;
      MinDots: Boolean;
      RIcon: TRect;
      mIconSize: Integer;

      Ctl3D, GenDots, FMinDots :boolean;
      Color, DotsColor: TColor;

begin
   Ctl3D       := not boolean(SearchParam(PRec, 'Ctl3D').Value^);
   GenDots     := not boolean(SearchParam(PRec, 'GenDots').Value^);
   FMinDots     := not boolean(SearchParam(PRec, 'MinDots').Value^);
   Color       := ColorRGB(integer(SearchParam(PRec, 'Color').Value^));
   DotsColor   := ColorRGB(integer(SearchParam(PRec, 'DotsColor').Value^));

   with PLoc(ed)^ do begin
      DotWidth :=(MaxDotWidth * Longint(FClientRect.Right - FClientRect.Left)) div HRes;
      DotHeight := Longint(DotWidth) * AspectV div AspectH;
      if DotHeight < MinDotHeight then DotHeight := MinDotHeight;
      if DotWidth < MinDotWidth then DotWidth := MinDotWidth;
      DotCenter.X := DotWidth div 2;
      DotCenter.Y := DotHeight div 2;
      InflateRect(FClientRect, -DotCenter.Y, -DotCenter.X);

      FClockCenter.X := (FClientRect.Right - FClientRect.Left) div 2;
      FClockCenter.Y :=(FClientRect.Bottom - FClientRect.Top) div 2;
      FClockRadius:=Min(FClockCenter.X, FClockCenter.Y) - Max(DotWidth, DotHeight);

      InflateRect(FclientRect, DotCenter.Y, DotCenter.X);
      SaveBrush := DB.Canvas.Brush.Color;
      SavePen := DB.Canvas.Pen.Color;
   TRY
      DB.Canvas.Brush.Color := DB.Canvas.Pen.Color;
      MinDots := ((DotWidth > MinDotWidth) and (DotHeight > MinDotHeight)) and FMinDots;
      for Pos := 0 to HandPositions - 1 do begin
         R.Top := (CircleTab^[Pos].Y *  FClockRadius) div CirTabScale + FClockCenter.Y;
         R.Left := (CircleTab^[Pos].X * FClockRadius) div CirTabScale + FClockCenter.X;
         if (Pos mod 5) <> 0 then begin
            if MinDots then begin
               if Ctl3D then begin
                  DB.Canvas.Brush.Color := clBtnShadow;
                  R.Right := R.Left + 2;
                  R.Bottom := R.Top + 2;
                  DB.Canvas.FillRect(R);
                  DB.Canvas.Brush.Color := clBtnHighlight;
                  OffsetRect(R, 1, 1);
                  DB.Canvas.FillRect(R);
               end;
               R.Right := R.Left + 1;
               R.Bottom := R.Top + 1;
               DB.Canvas.Brush.Color := clBtnHighlight;
               DB.Canvas.FillRect(R);
            end;
            DB.Canvas.Brush.Color := Color;
         end else begin
            R.Right := R.Left + DotWidth;
            R.Bottom := R.Top + DotHeight;
            OffsetRect(R, -DotCenter.X, -DotCenter.Y);
            if (Ctl3D and MinDots) and GenDots then
               with DB.Canvas^ do begin
                  Brush.Color := DotsColor;
                  Brush.BrushStyle := bsSolid;
                  FillRect(R);
               end;
            DB.Canvas.Brush.Color := DotsColor;
            if (not (Ctl3D and MinDots)) and GenDots then DB.Canvas.FillRect(R);
         end;
      end;
      if not ClockBmp.Empty then begin
         mIconSize := max(MinSizeIcon, Round((FClientRect.Right - FClientRect.Left) / FactorSizeIcon));
         if mIconSize mod 2 <> 0 then inc(mIconSize);
         RIcon.Top := FClockCenter.Y - (mIconSize div 2);
         RIcon.Bottom := RIcon.Top + mIconSize;
         RIcon.Right := FClockCenter.X  + Round(4 * FClockRadius / 5) + (mIconSize div 2);
         RIcon.Left := RIcon.Right - mIconSize;
         SetStretchBltMode(DB.Canvas.Handle, HALFTONE);
         ClockBmp.StretchDrawTransparent(DB.Canvas.Handle, RIcon, ClockBmp.Pixels[0,0]);
   end;
   FINALLY
      DB.Canvas.Brush.Color := SaveBrush;
      DB.Canvas.Pen.Color := SavePen;
   END;
   end;
end;

function HourHandPos(T: TClockTime): Integer;
begin
   if T.Hour >= 12 then Dec(T.Hour, 12);
   Result := (T.Hour * 5) + (T.Minute div 12);
end;

procedure DrawFatHand(PRec:PParamRec; ed:pointer; Pos: Integer; HourHand: Boolean);
var   ptSide, ptTail, ptTip: TPoint;
      Index, Hand: Integer;
      Scale: Longint;
      SaveMode: TPenMode;
      SaveBrush: TColor;
      SavePen: TColor;
      ColorPen: TColor;
      ColorPen2: TColor;
      ColorBrush: TColor;

      FHourColor, FMinuteColor, FShadowColor: TColor;
      HourSide, MinuteSide, HourTip, MinuteTip, HourTail, MinuteTail: integer;
      FShadeColor, FShadow: boolean;
      FTypeArrows:TTypeArrows;

      function GetShadeColor(Color: TColor; Shade: Byte) : TColor;
      var   fFrom: TRGB;
      begin
         PColor(@fFrom)^:= Color2RGB(Color);
         Result := RGB(
            Max(0, FFrom.R - Shade),
            Max(0, FFrom.G - Shade),
            Max(0, FFrom.B - Shade)
         );
      end;

begin
   FHourColor   := ColorRGB(integer(SearchParam(PRec, 'HourColor').Value^));
   FMinuteColor := ColorRGB(integer(SearchParam(PRec, 'MinuteColor').Value^));
   FShadowColor := ColorRGB(integer(SearchParam(PRec, 'ShadowColor').Value^));
   FShadeColor  := not boolean(SearchParam(PRec, 'ShadeColor').Value^);
   FShadow      := not boolean(SearchParam(PRec, 'Shadow').Value^);
   HourSide     := integer(SearchParam(PRec, 'HourSide').Value^);
   MinuteSide   := integer(SearchParam(PRec, 'MinuteSide').Value^);
   HourTip      := integer(SearchParam(PRec, 'HourTip').Value^);
   MinuteTip    := integer(SearchParam(PRec, 'MinuteTip').Value^);
   HourTail     := integer(SearchParam(PRec, 'HourTail').Value^);
   MinuteTail   := integer(SearchParam(PRec, 'MinuteTail').Value^);
   FTypeArrows  := TTypeArrows(integer(SearchParam(PRec, 'TypeArrows').Value^));

   with PLoc(ed)^ do begin
      if HourHand then Hand := HourSide else Hand := MinuteSide;
      Scale := (FClockRadius * Hand) div 100;
      Index := (Pos + SideShift) mod HandPositions;
      ptSide.Y := (CircleTab^[Index].Y * Scale) div CirTabScale;
      ptSide.X := (CircleTab^[Index].X * Scale) div CirTabScale;
      if HourHand then Hand := HourTip else Hand := MinuteTip;
      Scale := (FClockRadius * Hand) div 100;
      ptTip.Y := (CircleTab^[Pos].Y * Scale) div CirTabScale;
      ptTip.X := (CircleTab^[Pos].X * Scale) div CirTabScale;
      if HourHand then Hand := HourTail else Hand := MinuteTail;
      Scale := (FClockRadius * Hand) div 100;
      Index := (Pos + TailShift) mod HandPositions;
      ptTail.Y := (CircleTab^[Index].Y * Scale) div CirTabScale;
      ptTail.X := (CircleTab^[Index].X * Scale) div CirTabScale;
      with DB.Canvas^ do begin
         SaveMode := Pen.PenMode;
         Pen.PenMode := pmCopy;
         SaveBrush := Brush.Color;
         SavePen := Pen.Color;
      TRY
         if HourHand then
            Pen.Color := FHourColor
         else
            Pen.Color := FMinuteColor;

         ColorPen := Pen.Color;
         if FShadeColor then begin
            Brush.Color := GetShadeColor(Color2RGB(Pen.Color), 60 - Depth[Pos]);
            Pen.Color := Brush.Color;
         end else
            Brush.Color := Pen.Color;
         ColorPen2 := Pen.Color;
         ColorBrush := Brush.Color;

         if FTypeArrows = arEdge then begin
            if FShadow then begin
               Pen.Color := FShadowColor;
               Brush.Color := Pen.Color;
               Polygon([MakePoint(FClockCenter.X - ptSide.X + 1, FClockCenter.Y - ptSide.Y + 1),
                        MakePoint(FClockCenter.X + ptTip.X + 1, FClockCenter.Y + ptTip.Y + 1),
                        MakePoint(FClockCenter.X + ptSide.X + 1, FClockCenter.Y + ptSide.Y + 1),
                        MakePoint(FClockCenter.X + ptTail.X + 1, FClockCenter.Y + ptTail.Y + 1),
                        MakePoint(FClockCenter.X - ptSide.X + 1, FClockCenter.Y - ptSide.Y + 1)]);
               Pen.Color := ColorPen2;
               Brush.Color := ColorBrush;
            end;
            Polygon([MakePoint(FClockCenter.X - ptSide.X, FClockCenter.Y - ptSide.Y),
                     MakePoint(FClockCenter.X + ptTip.X, FClockCenter.Y + ptTip.Y),
                     MakePoint(FClockCenter.X + ptTail.X, FClockCenter.Y + ptTail.Y),
                     MakePoint(FClockCenter.X - ptSide.X, FClockCenter.Y - ptSide.Y)]);
         end else if FTypeArrows = arRectangle then begin
            if FShadow then begin
               Pen.Color := FShadowColor;
               Brush.Color := Pen.Color;
               Polygon([MakePoint(FClockCenter.X - ptSide.X + ptTail.X + 1, FClockCenter.Y - ptSide.Y + ptTail.Y + 1),
                        MakePoint(FClockCenter.X - ptSide.X + ptTip.X + 1, FClockCenter.Y - ptSide.Y + ptTip.Y + 1),
                        MakePoint(FClockCenter.X + ptSide.X + ptTip.X + 1, FClockCenter.Y + ptSide.Y + ptTip.Y + 1),
                        MakePoint(FClockCenter.X + ptSide.X + ptTail.X + 1, FClockCenter.Y + ptSide.Y + ptTail.Y + 1),
                        MakePoint(FClockCenter.X - ptSide.X + ptTail.X + 1, FClockCenter.Y - ptSide.Y + ptTail.Y + 1)]);
               Pen.Color := ColorPen2;
               Brush.Color := ColorBrush;
            end;
            Polygon([MakePoint(FClockCenter.X - ptSide.X + ptTail.X, FClockCenter.Y - ptSide.Y + ptTail.Y),
                     MakePoint(FClockCenter.X - ptSide.X + ptTip.X, FClockCenter.Y - ptSide.Y + ptTip.Y),
                     MakePoint(FClockCenter.X + ptTip.X, FClockCenter.Y + ptTip.Y),
                     MakePoint(FClockCenter.X + ptTail.X, FClockCenter.Y + ptTail.Y),
                     MakePoint(FClockCenter.X - ptSide.X + ptTail.X, FClockCenter.Y - ptSide.Y + ptTail.Y)]);
         end else begin
            if FShadow then begin
               Pen.Color := FShadowColor;
               Brush.Color := Pen.Color;
               Polygon([MakePoint(FClockCenter.X - ptSide.X + ptTail.X + 1, FClockCenter.Y - ptSide.Y + ptTail.Y + 1),
                        MakePoint(FClockCenter.X + ptTip.X + 1, FClockCenter.Y + ptTip.Y + 1),
                        MakePoint(FClockCenter.X + ptSide.X + ptTail.X + 1, FClockCenter.Y + ptSide.Y + ptTail.Y + 1),
                        MakePoint(FClockCenter.X - ptSide.X + ptTail.X + 1, FClockCenter.Y - ptSide.Y + ptTail.Y + 1)]);
               Pen.Color := ColorPen2;
               Brush.Color := ColorBrush;
            end;
            Polygon([MakePoint(FClockCenter.X - ptSide.X + ptTail.X, FClockCenter.Y - ptSide.Y + ptTail.Y),
                     MakePoint(FClockCenter.X + ptTip.X, FClockCenter.Y + ptTip.Y),
                     MakePoint(FClockCenter.X + ptTail.X, FClockCenter.Y + ptTail.Y),
                     MakePoint(FClockCenter.X - ptSide.X + ptTail.X, FClockCenter.Y - ptSide.Y + ptTail.Y)]);
         end;

         if FShadeColor then begin
            Pen.Color := ColorPen;
            Brush.Color := GetShadeColor(Color2RGB(Pen.Color), Depth[Pos]);
            Pen.Color := Brush.Color;
         end else
            Brush.Color := Pen.Color;

         if FTypeArrows = arEdge then begin
            Polygon([MakePoint(FClockCenter.X + ptSide.X, FClockCenter.Y + ptSide.Y),
                     MakePoint(FClockCenter.X + ptTip.X, FClockCenter.Y + ptTip.Y),
                     MakePoint(FClockCenter.X + ptTail.X, FClockCenter.Y + ptTail.Y),
                     MakePoint(FClockCenter.X + ptSide.X, FClockCenter.Y + ptSide.Y)]);
         end else if FTypeArrows = arRectangle then begin
            Polygon([MakePoint(FClockCenter.X + ptSide.X + ptTail.X, FClockCenter.Y + ptSide.Y + ptTail.Y),
                     MakePoint(FClockCenter.X + ptSide.X + ptTip.X, FClockCenter.Y + ptSide.Y + ptTip.Y),
                     MakePoint(FClockCenter.X + ptTip.X, FClockCenter.Y + ptTip.Y),
                     MakePoint(FClockCenter.X + ptTail.X, FClockCenter.Y + ptTail.Y),
                     MakePoint(FClockCenter.X + ptSide.X + ptTail.X, FClockCenter.Y + ptSide.Y + ptTail.Y)]);
         end else begin
            Polygon([MakePoint(FClockCenter.X + ptSide.X + ptTail.X, FClockCenter.Y + ptSide.Y + ptTail.Y),
                     MakePoint(FClockCenter.X + ptTip.X, FClockCenter.Y + ptTip.Y),
                     MakePoint(FClockCenter.X + ptTail.X, FClockCenter.Y + ptTail.Y),
                     MakePoint(FClockCenter.X + ptSide.X + ptTail.X, FClockCenter.Y + ptSide.Y + ptTail.Y)]);
         end;
      FINALLY
         Pen.Color := SavePen;
         Pen.PenMode := SaveMode;
         Brush.Color := SaveBrush;
      END;
      end;
   end;
end;

procedure DrawSecondHand(PRec:PParamRec; ed:pointer; Pos: Integer);
var   Radius: Longint;
      RadiusTail: Longint;
      SaveMode: TPenMode;
      SavePen: TColor;
      SavePenWidth: Integer;
      SaveBrush: TColor;

      FShowSecond, FCenter:boolean;
      FSecondColor: TColor;
      SecondWidth: integer;
      SecondTip, SecondTail: integer;

begin

   FShowSecond  := not boolean(SearchParam(PRec, 'ShowSeconds').Value^);
   FCenter      := not boolean(SearchParam(PRec, 'Center').Value^);
   FSecondColor := ColorRGB(integer(SearchParam(PRec, 'SecondColor').Value^));
   SecondWidth  := integer(SearchParam(PRec, 'SecondWidth').Value^);
   SecondTip    := integer(SearchParam(PRec, 'SecondTip').Value^);
   SecondTail   := integer(SearchParam(PRec, 'SecondTail').Value^);

   with PLoc(ed)^ do begin
      Radius := (FClockRadius * SecondTip) div 100;
      RadiusTail := (FClockRadius * SecondTail) div 100;
      SavePen := DB.Canvas.Pen.Color;
      SaveBrush := DB.Canvas.Brush.Color;
      DB.Canvas.Pen.Color := FSecondColor;
      SaveMode := DB.Canvas.Pen.PenMode;
      DB.Canvas.Pen.PenMode := pmCopy;
      SavePenWidth := DB.Canvas.Pen.PenWidth;
      DB.Canvas.Pen.PenWidth := SecondWidth;
   TRY
      if FShowSecond then begin
         DB.Canvas.MoveTo(FClockCenter.X - ((CircleTab^[Pos].X * RadiusTail) div
                          CirTabScale), FClockCenter.Y - ((CircleTab^[Pos].Y * RadiusTail) div
                          CirTabScale));
         DB.Canvas.LineTo(FClockCenter.X + ((CircleTab^[Pos].X * Radius) div
                          CirTabScale), FClockCenter.Y + ((CircleTab^[Pos].Y * Radius) div
                          CirTabScale));
      end;
      if FCenter then begin
         DB.Canvas.Brush.Color := DB.Canvas.Pen.Color;
         DB.Canvas.Pen.PenWidth := max(3, SecondWidth);
         DB.Canvas.Ellipse(FClockCenter.X - max(3, SecondWidth), FClockCenter.Y - max(3, SecondWidth),
                           FClockCenter.X + max(3, SecondWidth), FClockCenter.Y + max(3, SecondWidth));
      end;
   FINALLY
      DB.Canvas.Pen.PenMode := SaveMode;
      DB.Canvas.Pen.Color := SavePen;
      DB.Canvas.Pen.PenWidth := SavePenWidth;
      DB.Canvas.Brush.Color := SaveBrush;
   END;
   end;
end;

procedure Init(PRec:PParamRec; var ed:pointer; DTools:PDrawTools); cdecl;
var   Pos: Integer;
      vSize: Integer;
      hSize: Integer;
      DC: HDC;
      p:PLoc;
      w, h: cardinal;
begin
   new(p);
   ed := p;
   with p^ do begin
      DC := GetDC(0);
   TRY
      VRes  := GetDeviceCaps(DC, VERTRES);
      HRes  := GetDeviceCaps(DC, HORZRES);
      vSize := GetDeviceCaps(DC, VERTSIZE);
      hSize := GetDeviceCaps(DC, HORZSIZE);
   FINALLY
      ReleaseDC(0, DC);
   END;
      AspectV := (Longint(VRes) * MmPerDm) div Longint(vSize);
      AspectH := (Longint(HRes) * MmPerDm) div Longint(hSize);
      CircleTab := PPointArray(@ClockData);
      for Pos := 0 to HandPositions - 1 do CircleTab^[Pos].Y := Longint(CircleTab^[Pos].Y) * AspectV div AspectH;
      FDispTime := GetTime(PRec, ed);
      DB := NewBitmap(integer(SearchParam(PRec, 'Width').Value^), integer(SearchParam(PRec, 'Height').Value^));
      DT := DTools;
      PictureBack := DT.CreateBitmap(SearchParam(PRec, 'PictureBack'));
      DT.GetSizeBitmap(PictureBack, w, h);
      BackBmp := NewBitmap(w, h);
      if (w <> 0) and (h <> 0) then DT.DrawBitmap(PictureBack, BackBmp.Canvas.Handle, 0, 0);
      PictureClock := DT.CreateBitmap(SearchParam(PRec, 'PictureClock'));
      DT.GetSizeBitmap(PictureClock, w, h);
      ClockBmp := NewBitmap(w, h);
      if (w <> 0) and (h <> 0) then DT.DrawBitmap(PictureClock, ClockBmp.Canvas.Handle, 0, 0);
   end;
end;

procedure Close(PRec:PParamRec; ed:pointer); cdecl;
var   p:PLoc;
begin
   p := PLoc(ed);
   p.DB.Free;
   p.BackBmp.free;
   p.ClockBmp.free;
   DT.DeleteBitmap(p.PictureBack);
   dispose(p);
end;

procedure Change(PRec:PParamRec; ed:pointer; Index:integer); cdecl;
var   w, h: cardinal;
begin
   if PRec[Index].Name = 'Width' then
      PLoc(ed).DB.Width := integer(SearchParam(PRec, 'Width').Value^)
   else if PRec[Index].Name = 'Height' then
      PLoc(ed).DB.Height := integer(SearchParam(PRec, 'Height').Value^)
   else if PRec[Index].Name = 'PictureBack' then
      with PLoc(ed)^ do begin
         DT.DeleteBitmap(PictureBack);
         PictureBack := DT.CreateBitmap(@PRec[Index]);
         BackBmp.Clear;
         DT.GetSizeBitmap(PictureBack, w, h);
         BackBmp.Width  := w;
         BackBmp.Height := h;
         if (w <> 0) and (h <> 0) then DT.DrawBitmap(PictureBack, BackBmp.Canvas.Handle, 0, 0);
      end
   else if PRec[Index].Name = 'PictureClock' then
      with PLoc(ed)^ do begin
         DT.DeleteBitmap(PictureClock);
         PictureClock := DT.CreateBitmap(@PRec[Index]);
         ClockBmp.Clear;
         DT.GetSizeBitmap(PictureClock, w, h);
         ClockBmp.Width  := w;
         ClockBmp.Height := h;
         if (w <> 0) and (h <> 0) then DT.DrawBitmap(PictureClock, ClockBmp.Canvas.Handle, 0, 0);
      end
   else if PRec[Index].Name = 'HourOffset' then
      PLoc(ed).FDispTime := GetTime(PRec, ed);
end;

procedure Draw(PRec:PParamRec; ed:pointer; dc:HDC); cdecl;
var   Color: TColor;

      k: Real;
      r: TRect;

      procedure Fill(x1,y1,x2,y2:integer);
      var   ra:TRect;
      begin
         ra.Left   := x1;
         ra.Top    := y1;
         ra.Right  := x2;
         ra.Bottom := y2;
         PLoc(ed).DB.Canvas.FillRect(ra);
      end;

begin
   Color := ColorRGB(integer(SearchParam(PRec, 'Color').Value^));
   with PLoc(ed)^ do begin
      FClientRect.Left  := 0;
      FClientRect.Top   := 0;
      FClientRect.Right := integer(SearchParam(PRec, 'Width').Value^);
      FClientRect.Bottom:= integer(SearchParam(PRec, 'Height').Value^);
      DB.Canvas.Brush.Color := Color;
      SetBkMode(DB.Canvas.Handle, Windows.TRANSPARENT);
      DB.Canvas.FillRect(FClientRect);
      if not BackBmp.Empty then begin
         r := FClientRect;
         k := BackBmp.Height/BackBmp.Width;
         if ((FClientRect.Bottom - FClientRect.Top) < (FClientRect.Right - FClientRect.Left)*k) = true then begin
            r.Top := Round(((FClientRect.Bottom - FClientRect.Top) - (FClientRect.Right - FClientRect.Left)*k)/2);
            r.Bottom := FClientRect.Bottom - FClientRect.Top - r.Top;
            Fill(0, 0, FClientRect.Right - FClientRect.Left, r.top);
            Fill(0, r.Bottom, FClientRect.Right - FClientRect.Left, FClientRect.Bottom - FClientRect.Top);
         end else begin
            r.Left := Round(((FClientRect.Right - FClientRect.Left) - (FClientRect.Bottom - FClientRect.Top)/k)/2);
            r.Right := FClientRect.Right - FClientRect.Left - r.Left;
            Fill(0, 0, r.left, FClientRect.Bottom - FClientRect.Top);
            Fill(r.Right, 0, FClientRect.Right - FClientRect.Left, FClientRect.Bottom - FClientRect.Top);
         end;
         SetStretchBltMode(DB.Canvas.Handle, HALFTONE);
         BackBmp.StretchDraw(DB.Canvas.Handle, FClientRect);
      end;
      DrawAnalogFace(PRec, ed);
      DrawFatHand(PRec, ed, HourHandPos(FDispTime), True);
      DrawFatHand(PRec, ed, FDispTime.Minute, False);
      DrawSecondHand(PRec, ed, FDispTime.Second);
      DB.Canvas.Pen.Color := DB.Canvas.Brush.Color;
      DB.Draw(DC, 0, 0);
   end;
end;

exports
   Init,
   Close,
   Draw,
   Change;

begin
end.
 