unit hiClock;

interface

uses Messages, Windows, kol, Share, Debug, Win;

type
  TTypeArrows = (arEdge, arRectangle, arTriangle);

  TClockTime = packed record
    Hour, Minute, Second: Word;
  end;

  TOnGetClockTime = procedure(Sender: PControl; Time:TClockTime) of object;

type
  PPointArray = ^TPointArray;
  TPointArray = array [0..60 * 2 - 1] of TSmallPoint;

type
  PPoints = ^TPoints;
  TPoints = array[0..0] of TPoint;

const
  ClockData: array[0..60 * 4 - 1] of Byte = (
    $00, $00, $C1, $E0, $44, $03, $EC, $E0, $7F, $06, $6F, $E1,
    $A8, $09, $48, $E2, $B5, $0C, $74, $E3, $9F, $0F, $F0, $E4,
    $5E, $12, $B8, $E6, $E9, $14, $C7, $E8, $39, $17, $17, $EB,
    $48, $19, $A2, $ED, $10, $1B, $60, $F0, $8C, $1C, $4B, $F3,
    $B8, $1D, $58, $F6, $91, $1E, $81, $F9, $14, $1F, $BC, $FC,
    $40, $1F, $00, $00, $14, $1F, $44, $03, $91, $1E, $7F, $06,
    $B8, $1D, $A8, $09, $8C, $1C, $B5, $0C, $10, $1B, $A0, $0F,
    $48, $19, $5E, $12, $39, $17, $E9, $14, $E9, $14, $39, $17,
    $5E, $12, $48, $19, $9F, $0F, $10, $1B, $B5, $0C, $8C, $1C,
    $A8, $09, $B8, $1D, $7F, $06, $91, $1E, $44, $03, $14, $1F,
    $00, $00, $3F, $1F, $BC, $FC, $14, $1F, $81, $F9, $91, $1E,
    $58, $F6, $B8, $1D, $4B, $F3, $8C, $1C, $60, $F0, $10, $1B,
    $A2, $ED, $48, $19, $17, $EB, $39, $17, $C7, $E8, $E9, $14,
    $B8, $E6, $5E, $12, $F0, $E4, $9F, $0F, $74, $E3, $B5, $0C,
    $48, $E2, $A8, $09, $6F, $E1, $7F, $06, $EC, $E0, $44, $03,
    $C1, $E0, $00, $00, $EC, $E0, $BC, $FC, $6F, $E1, $81, $F9,
    $48, $E2, $58, $F6, $74, $E3, $4B, $F3, $F0, $E4, $60, $F0,
    $B8, $E6, $A2, $ED, $C7, $E8, $17, $EB, $17, $EB, $C7, $E8,
    $A2, $ED, $B8, $E6, $61, $F0, $F0, $E4, $4B, $F3, $74, $E3,
    $58, $F6, $48, $E2, $81, $F9, $6F, $E1, $BC, $FC, $EC, $E0);

const
  Depth: array[0..59] of Byte = (
    40, 42, 44, 46, 48, 50, 52, 54, 56, 58,
    60, 58, 56, 54, 52, 50, 48, 46, 44, 42,
    40, 38, 36, 34, 32, 30, 28, 26, 24, 22,
    20, 18, 16, 14, 12, 10,  8,  6,  4,  2,
     0,  2,  4,  6,  8, 10, 12, 14, 16, 18,
    20, 22, 24, 26, 28, 30, 32, 34, 36, 38);

const
  MaxDotWidth    = 25; { maximum Hour-marking dot width  }
  MinDotWidth    = 2;  { minimum Hour-marking dot width  }
  MinDotHeight   = 1;  { minimum Hour-marking dot height }
  MinSizeIcon    = 4;  { minimum IconSize }
  FactorSizeIcon = 200 / 24;

  { conversion factors }
  CirTabScale = 8000; { circle table values scale down value  }
  MmPerDm     = 100;  { millimeters per decimeter             }

  { number of hand positions on... }
  HandPositions = 60;                    { ...entire clock         }
  SideShift     = (HandPositions div 4); { ...90 degrees of clock  }
  TailShift     = (HandPositions div 2); { ...180 degrees of clock }

type
 THiClock = class(THIWin)
   private
    CircleTab: PPointArray;
    HRes: Integer;    { width of the display (in pixels)                    }
    VRes: Integer;    { height of the display (in raster lines)             }
    AspectH: Longint; { number of pixels per decimeter on the display       }
    AspectV: Longint; { number of raster lines per decimeter on the display }

    FTimer: PTimer;
    FDispTime: TClockTime;
    FClientRect:Trect;
    DB: PBitmap;
    BackBmp: PBitmap;
    ClockBmp: PBitmap;
    FShowSecond:Boolean;
    FClockRadius: Longint;
    FHourOffset: integer;
    FClockCenter: TPoint;
    FDotsColor: TColor;
    FHourColor: TColor;
    FMinuteColor: TColor;
    FSecondColor: TColor;
    FShadowColor: TColor;
    FShadeColor: boolean;
    FMinDots: boolean;
    FGenDots: boolean;    
    FCenter: boolean;
    FShadow: boolean;    
    FTypeArrows: TTypeArrows;
    procedure SetPicture(Value:HBITMAP);
    procedure OnTimer(Sender: PObj);
    procedure PaintAnalogClock;
    procedure DrawFatHand(Pos: Integer; HourHand: Boolean);
    procedure ClockOnPaint( Sender: PControl; DC: HDC ) ;
    procedure DrawSecondHand(Pos: Integer);
    procedure DrawAnalogFace;
    function  GetTime:TClockTime;
    procedure SetPictureClock(Value:HBITMAP);    
   public
    _prop_EdgeStyle: TEdgeStyle;
    _prop_SecondWidth: Integer;    
    _prop_HourSide: Integer;
    _prop_MinuteSide: Integer;
    _prop_HourTip: Integer;
    _prop_MinuteTip: Integer;
    _prop_SecondTip: Integer;
    _prop_HourTail: Integer;
    _prop_MinuteTail: Integer;
    _prop_SecondTail: Integer;

    //   
    property _prop_ShowSeconds: Boolean write FShowSecond ;//default True;
    //   
    property _prop_DotsColor: Tcolor write FDotsColor ;
    //   
    property _prop_HourColor: Tcolor write FHourColor ;
    //   
    property _prop_MinuteColor: Tcolor write FMinuteColor ;
    //   
    property _prop_ShadeColor: Boolean write FShadeColor ;
    //   
    property _prop_SecondColor: Tcolor write FSecondColor ;
    //   
    property _prop_GenDots: boolean write FGenDots ;
    //   
    property _prop_MinDots: boolean write FMinDots ;
    //  
    property _prop_Center: boolean write FCenter ;
    //    
    property _prop_Shadow: boolean write FShadow ;
    //    
    property _prop_ShadowColor: Integer write FShadowColor ;    
    //  
    property _prop_PictureBack:HBITMAP write SetPicture;
    //    
    property _prop_PictureClock:HBITMAP write SetPictureClock;
    //  
    property _prop_TypeArrows:TTypeArrows write FTypeArrows;
    //  
    property _prop_HourOffset:integer write FHourOffset;

    constructor Create(Paren:PControl);
    destructor Destroy; override;
    procedure Init; override;
    procedure _work_doStopClock(var _Data:TData; Index:word);
    procedure _work_doPictureBack(var _Data:TData; Index:word);
    procedure _work_doPictureClock(var _Data:TData; Index:word);
    procedure _work_doMinDots(var _Data:TData; Index:word);
    procedure _work_doGenDots(var _Data:TData; Index:word);    
    procedure _work_doDotsColor(var _Data:TData; Index:word);
    procedure _work_doHourColor(var _Data:TData; Index:word);
    procedure _work_doHourOffset(var _Data:TData; Index:word);    
    procedure _work_doMinuteColor(var _Data:TData; Index:word);
    procedure _work_doSecondColor(var _Data:TData; Index:word);    
    procedure _work_doShadeColor(var _Data:TData; Index:word);
    procedure _work_doCenter(var _Data:TData; Index:word);    
    procedure _work_doShadow(var _Data:TData; Index:word);
    procedure _work_doShadowColor(var _Data:TData; Index:word);
    procedure _work_doShowSeconds(var _Data:TData; Index:word);    
    procedure _work_doTypeArrows(var _Data:TData; Index:word);                                  
 end;
implementation

procedure THiClock._work_doStopClock;begin FTimer.Enabled := not ReadBool(_Data); end;
procedure THiClock._work_doGenDots;begin FGenDots := ReadBool(_Data); Control.Invalidate; end;
procedure THiClock._work_doMinDots;begin FMinDots := ReadBool(_Data); Control.Invalidate; end;
procedure THiClock._work_doDotsColor;begin FDotsColor := ToInteger(_Data); Control.Invalidate; end;
procedure THiClock._work_doHourColor;begin FHourColor := ToInteger(_Data); Control.Invalidate; end;
procedure THiClock._work_doMinuteColor;begin FMinuteColor := ToInteger(_Data); Control.Invalidate; end;
procedure THiClock._work_doSecondColor;begin FSecondColor := ToInteger(_Data); Control.Invalidate; end;
procedure THiClock._work_doShadowColor;begin FShadowColor := ToInteger(_Data); Control.Invalidate; end;
procedure THiClock._work_doHourOffset;begin FHourOffset := ToInteger(_Data); Control.Invalidate; end;
procedure THiClock._work_doShowSeconds;begin FShowSecond := ReadBool(_Data); Control.Invalidate; end;
procedure THiClock._work_doCenter;begin FCenter := ReadBool(_Data); Control.Invalidate; end;
procedure THiClock._work_doShadeColor;begin FShadeColor := ReadBool(_Data); Control.Invalidate; end;
procedure THiClock._work_doShadow;begin FShadow := ReadBool(_Data); Control.Invalidate; end;
procedure THiClock._work_doTypeArrows;begin FTypeArrows := TTypeArrows(ToInteger(_Data)); Control.Invalidate; end;

procedure THiClock.SetPicture;
begin
   if Value = 0 then exit;
   BackBmp.Handle := CopyImage(Value, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);
end;

procedure THiClock.SetPictureClock;
begin
   if Value = 0 then exit;
   ClockBmp.Handle := CopyImage(Value, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION); 
end;

procedure THiClock._work_doPictureBack;
begin
   BackBmp.Clear;
   if not _IsBitmap(_Data) then exit;
   SetPicture(ToBitmap(_Data).Handle);
   Control.Invalidate;
end;

procedure THiClock._work_doPictureClock;
begin
   ClockBmp.Clear;
   if not _IsBitmap(_Data) then exit;
   SetPictureClock(ToBitmap(_Data).Handle);
   Control.Invalidate;
end;

procedure THiClock.DrawAnalogFace;
var   Pos, DotHeight, DotWidth: Integer;
      DotCenter: TPoint;
      R: TRect;
      SaveBrush, SavePen: TColor;
      MinDots: Boolean;
      RIcon: TRect;      
      mIconSize: Integer;
begin
   DotWidth :=(MaxDotWidth * Longint(FClientRect.Right - FClientRect.Left)) div HRes;
   DotHeight := Longint(DotWidth) * AspectV div AspectH;
   
   if DotHeight < MinDotHeight then DotHeight := MinDotHeight;
   if DotWidth < MinDotWidth then DotWidth := MinDotWidth;
   DotCenter.X := DotWidth div 2;
   DotCenter.Y := DotHeight div 2;
   InflateRect(FClientRect, -DotCenter.Y, -DotCenter.X);

   FClockCenter.X := (FClientRect.Right - FClientRect.Left) div 2;
   FClockCenter.Y :=(FClientRect.Bottom - FClientRect.Top) div 2;
   FClockRadius:=Min(FClockCenter.X, FClockCenter.Y) - Max(DotWidth, DotHeight);

   InflateRect(FclientRect, DotCenter.Y, DotCenter.X);
   SaveBrush := DB.Canvas.Brush.Color;
   SavePen := DB.Canvas.Pen.Color;
TRY
   DB.Canvas.Brush.Color := DB.Canvas.Pen.Color;
   MinDots := ((DotWidth > MinDotWidth) and (DotHeight > MinDotHeight)) and FMinDots;
   for Pos := 0 to HandPositions - 1 do begin
      R.Top := (CircleTab^[Pos].Y *  FClockRadius) div CirTabScale + FClockCenter.Y;
      R.Left := (CircleTab^[Pos].X * FClockRadius) div CirTabScale + FClockCenter.X;
      if (Pos mod 5) <> 0 then begin
         if MinDots then begin
            if Control.Ctl3D then begin
               DB.Canvas.Brush.Color := clBtnShadow;
               R.Right := R.Left + 2;
               R.Bottom := R.Top + 2;
               DB.Canvas.FillRect(R);
               DB.Canvas.Brush.Color := clBtnHighlight;
               OffsetRect(R, 1, 1);
               DB.Canvas.FillRect(R);
            end;
            R.Right := R.Left + 1;
            R.Bottom := R.Top + 1;
            DB.Canvas.Brush.Color := clBtnHighlight;
            DB.Canvas.FillRect(R);
         end;
         DB.Canvas.Brush.Color := Control.Color;
      end else begin
         R.Right := R.Left + DotWidth;
         R.Bottom := R.Top + DotHeight;
         OffsetRect(R, -DotCenter.X, -DotCenter.Y);
         DB.Canvas.Brush.Color := FDotsColor;
         if FGenDots then DB.Canvas.FillRect(R);
      end;
   end;
   if not ClockBmp.Empty then begin
      mIconSize := max(MinSizeIcon, Round((FClientRect.Right - FClientRect.Left) / FactorSizeIcon));
      if mIconSize mod 2 <> 0 then inc(mIconSize);
      RIcon.Top := FClockCenter.Y - (mIconSize div 2);
      RIcon.Bottom := RIcon.Top + mIconSize;
      RIcon.Right := FClockCenter.X  + Round(4 * FClockRadius / 5) + (mIconSize div 2);
      RIcon.Left := RIcon.Right - mIconSize;
      SetStretchBltMode(DB.Canvas.Handle, HALFTONE);
      ClockBmp.StretchDrawTransparent(DB.Canvas.Handle, RIcon, ClockBmp.Pixels[0,0]);
   end;
FINALLY
   DB.Canvas.Brush.Color := SaveBrush;
   DB.Canvas.Pen.Color := SavePen;
END;        
end;

procedure THiClock.DrawFatHand(Pos: Integer; HourHand: Boolean);
var   ptSide, ptTail, ptTip: TPoint;
      Index, Hand: Integer;
      Scale: Longint;
      SaveMode: TPenMode;
      SaveBrush: TColor;
      SavePen: TColor;
      ColorPen: TColor;
      ColorPen2: TColor;
      ColorBrush: TColor;
      
      function GetShadeColor(Color: TColor; Shade: Byte) : TColor;
      var   fFrom: TRGB;
      begin
         PColor(@fFrom)^:= Color2RGB(Color);
         Result := RGB(
            Max(0, FFrom.R - Shade),
            Max(0, FFrom.G - Shade),
            Max(0, FFrom.B - Shade)
         );
      end;

begin
   if HourHand then Hand := _prop_HourSide else Hand := _prop_MinuteSide;
   Scale := (FClockRadius * Hand) div 100;
   Index := (Pos + SideShift) mod HandPositions;
   ptSide.Y := (CircleTab^[Index].Y * Scale) div CirTabScale;
   ptSide.X := (CircleTab^[Index].X * Scale) div CirTabScale;
   if HourHand then Hand := _prop_HourTip else Hand := _prop_MinuteTip;
   Scale := (FClockRadius * Hand) div 100;
   ptTip.Y := (CircleTab^[Pos].Y * Scale) div CirTabScale;
   ptTip.X := (CircleTab^[Pos].X * Scale) div CirTabScale;
   if HourHand then Hand := _prop_HourTail else Hand := _prop_MinuteTail;
   Scale := (FClockRadius * Hand) div 100;
   Index := (Pos + TailShift) mod HandPositions;
   ptTail.Y := (CircleTab^[Index].Y * Scale) div CirTabScale;
   ptTail.X := (CircleTab^[Index].X * Scale) div CirTabScale;
   with DB.Canvas{$ifndef F_P}^{$endif} do begin
      SaveMode := Pen.PenMode;
      Pen.PenMode := pmCopy;
      SaveBrush := Brush.Color;
      SavePen := Pen.Color;
   TRY 
      if HourHand then 
         Pen.Color := FHourColor
      else
         Pen.Color := FMinuteColor;
                     
      ColorPen := Pen.Color;
      if FShadeColor then begin
         Brush.Color := GetShadeColor(Color2RGB(Pen.Color), 60 - Depth[Pos]);
         Pen.Color := Brush.Color;
      end else
         Brush.Color := Pen.Color;
      ColorPen2 := Pen.Color;
      ColorBrush := Brush.Color;

      if FTypeArrows = arEdge then begin
         if FShadow then begin
            Pen.Color := FShadowColor;
            Brush.Color := Pen.Color; 
            Polygon([MakePoint(FClockCenter.X - ptSide.X + 1, FClockCenter.Y - ptSide.Y + 1),
                     MakePoint(FClockCenter.X + ptTip.X + 1, FClockCenter.Y + ptTip.Y + 1),
                     MakePoint(FClockCenter.X + ptSide.X + 1, FClockCenter.Y + ptSide.Y + 1),                  
                     MakePoint(FClockCenter.X + ptTail.X + 1, FClockCenter.Y + ptTail.Y + 1),
                     MakePoint(FClockCenter.X - ptSide.X + 1, FClockCenter.Y - ptSide.Y + 1)]);
            Pen.Color := ColorPen2;
            Brush.Color := ColorBrush;
         end;
         Polygon([MakePoint(FClockCenter.X - ptSide.X, FClockCenter.Y - ptSide.Y),
                  MakePoint(FClockCenter.X + ptTip.X, FClockCenter.Y + ptTip.Y),
                  MakePoint(FClockCenter.X + ptTail.X, FClockCenter.Y + ptTail.Y),
                  MakePoint(FClockCenter.X - ptSide.X, FClockCenter.Y - ptSide.Y)]);
      end else if FTypeArrows = arRectangle then begin
         if FShadow then begin
            Pen.Color := FShadowColor;
            Brush.Color := Pen.Color; 
            Polygon([MakePoint(FClockCenter.X - ptSide.X + ptTail.X + 1, FClockCenter.Y - ptSide.Y + ptTail.Y + 1),
                     MakePoint(FClockCenter.X - ptSide.X + ptTip.X + 1, FClockCenter.Y - ptSide.Y + ptTip.Y + 1),
                     MakePoint(FClockCenter.X + ptSide.X + ptTip.X + 1, FClockCenter.Y + ptSide.Y + ptTip.Y + 1),
                     MakePoint(FClockCenter.X + ptSide.X + ptTail.X + 1, FClockCenter.Y + ptSide.Y + ptTail.Y + 1),                   
                     MakePoint(FClockCenter.X - ptSide.X + ptTail.X + 1, FClockCenter.Y - ptSide.Y + ptTail.Y + 1)]);
            Pen.Color := ColorPen2;
            Brush.Color := ColorBrush;
         end;
         Polygon([MakePoint(FClockCenter.X - ptSide.X + ptTail.X, FClockCenter.Y - ptSide.Y + ptTail.Y),
                  MakePoint(FClockCenter.X - ptSide.X + ptTip.X, FClockCenter.Y - ptSide.Y + ptTip.Y),
                  MakePoint(FClockCenter.X + ptTip.X, FClockCenter.Y + ptTip.Y),
                  MakePoint(FClockCenter.X + ptTail.X, FClockCenter.Y + ptTail.Y),
                  MakePoint(FClockCenter.X - ptSide.X + ptTail.X, FClockCenter.Y - ptSide.Y + ptTail.Y)]);
      end else begin
         if FShadow then begin
            Pen.Color := FShadowColor;
            Brush.Color := Pen.Color; 
            Polygon([MakePoint(FClockCenter.X - ptSide.X + ptTail.X + 1, FClockCenter.Y - ptSide.Y + ptTail.Y + 1),
                     MakePoint(FClockCenter.X + ptTip.X + 1, FClockCenter.Y + ptTip.Y + 1),
                     MakePoint(FClockCenter.X + ptSide.X + ptTail.X + 1, FClockCenter.Y + ptSide.Y + ptTail.Y + 1),                   
                     MakePoint(FClockCenter.X - ptSide.X + ptTail.X + 1, FClockCenter.Y - ptSide.Y + ptTail.Y + 1)]);
            Pen.Color := ColorPen2;
            Brush.Color := ColorBrush;
         end;
         Polygon([MakePoint(FClockCenter.X - ptSide.X + ptTail.X, FClockCenter.Y - ptSide.Y + ptTail.Y),
                  MakePoint(FClockCenter.X + ptTip.X, FClockCenter.Y + ptTip.Y),
                  MakePoint(FClockCenter.X + ptTail.X, FClockCenter.Y + ptTail.Y),
                  MakePoint(FClockCenter.X - ptSide.X + ptTail.X, FClockCenter.Y - ptSide.Y + ptTail.Y)]);
      end;

      if FShadeColor then begin
         Pen.Color := ColorPen;
         Brush.Color := GetShadeColor(Color2RGB(Pen.Color), Depth[Pos]);
         Pen.Color := Brush.Color;
      end else
         Brush.Color := Pen.Color;

      if FTypeArrows = arEdge then begin
         Polygon([MakePoint(FClockCenter.X + ptSide.X, FClockCenter.Y + ptSide.Y),
                  MakePoint(FClockCenter.X + ptTip.X, FClockCenter.Y + ptTip.Y),
                  MakePoint(FClockCenter.X + ptTail.X, FClockCenter.Y + ptTail.Y),
                  MakePoint(FClockCenter.X + ptSide.X, FClockCenter.Y + ptSide.Y)]);
      end else if FTypeArrows = arRectangle then begin
         Polygon([MakePoint(FClockCenter.X + ptSide.X + ptTail.X, FClockCenter.Y + ptSide.Y + ptTail.Y),
                  MakePoint(FClockCenter.X + ptSide.X + ptTip.X, FClockCenter.Y + ptSide.Y + ptTip.Y),
                  MakePoint(FClockCenter.X + ptTip.X, FClockCenter.Y + ptTip.Y),
                  MakePoint(FClockCenter.X + ptTail.X, FClockCenter.Y + ptTail.Y),
                  MakePoint(FClockCenter.X + ptSide.X + ptTail.X, FClockCenter.Y + ptSide.Y + ptTail.Y)]);
      end else begin
         Polygon([MakePoint(FClockCenter.X + ptSide.X + ptTail.X, FClockCenter.Y + ptSide.Y + ptTail.Y),
                  MakePoint(FClockCenter.X + ptTip.X, FClockCenter.Y + ptTip.Y),
                  MakePoint(FClockCenter.X + ptTail.X, FClockCenter.Y + ptTail.Y),
                  MakePoint(FClockCenter.X + ptSide.X + ptTail.X, FClockCenter.Y + ptSide.Y + ptTail.Y)]);
      end;
   FINALLY
      Pen.Color := SavePen;
      Pen.PenMode := SaveMode;
      Brush.Color := SaveBrush;
   END;
   end;  
end;

function HourHandPos(T: TClockTime): Integer;
begin
   if T.Hour >= 12 then Dec(T.Hour, 12);
   Result := (T.Hour * 5) + (T.Minute div 12);
end;

procedure THiClock.DrawSecondHand(Pos: Integer);
var   Radius: Longint;
      RadiusTail: Longint;
      SaveMode: TPenMode;
      SavePen: TColor;
      SavePenWidth: Integer;
      SaveBrush: TColor;
begin
   Radius := (FClockRadius * _prop_SecondTip) div 100;
   RadiusTail := (FClockRadius * _prop_SecondTail) div 100;   
   SavePen := DB.Canvas.Pen.Color;
   SaveBrush := DB.Canvas.Brush.Color; 
   DB.Canvas.Pen.Color := FSecondColor;    
   SaveMode := DB.Canvas.Pen.PenMode;
   DB.Canvas.Pen.PenMode := pmCopy;
   SavePenWidth := DB.Canvas.Pen.PenWidth;
   DB.Canvas.Pen.PenWidth := _prop_SecondWidth;   
TRY
   if FShowSecond then begin
      DB.Canvas.MoveTo(FClockCenter.X - ((CircleTab^[Pos].X * RadiusTail) div
                       CirTabScale), FClockCenter.Y - ((CircleTab^[Pos].Y * RadiusTail) div
                       CirTabScale));
      DB.Canvas.LineTo(FClockCenter.X + ((CircleTab^[Pos].X * Radius) div
                       CirTabScale), FClockCenter.Y + ((CircleTab^[Pos].Y * Radius) div
                       CirTabScale));
   end;
   if FCenter then begin
      DB.Canvas.Brush.Color := DB.Canvas.Pen.Color;
      DB.Canvas.Pen.PenWidth := max(3, _prop_SecondWidth); 
      DB.Canvas.Ellipse(FClockCenter.X - max(3, _prop_SecondWidth), FClockCenter.Y - max(3, _prop_SecondWidth),
                     FClockCenter.X + max(3, _prop_SecondWidth), FClockCenter.Y + max(3, _prop_SecondWidth));
   end;
FINALLY
   DB.Canvas.Pen.PenMode := SaveMode;
   DB.Canvas.Pen.Color := SavePen;
   DB.Canvas.Pen.PenWidth := SavePenWidth;
   DB.Canvas.Brush.Color := SaveBrush;
END;
end;

procedure THiClock.PaintAnalogClock;
var   k: Real;
      r: TRect;

      procedure Fill(x1,y1,x2,y2:integer);
      var   ra:TRect;
      begin
         ra.Left   := x1;
         ra.Top    := y1;
         ra.Right  := x2;
         ra.Bottom := y2;
         DB.Canvas.FillRect(ra);
      end;

begin
   DB.Canvas.Brush.Color := Control.Color;
   SetBkMode(DB.Canvas.Handle, Windows.TRANSPARENT);
   DB.Canvas.FillRect(FClientRect);
   if not BackBmp.Empty then begin
      r := FClientRect;
      k := BackBmp.Height/BackBmp.Width;
      if ((FClientRect.Bottom - FClientRect.Top) < (FClientRect.Right - FClientRect.Left)*k) = true then begin
         r.Top := Round(((FClientRect.Bottom - FClientRect.Top) - (FClientRect.Right - FClientRect.Left)*k)/2);
         r.Bottom := FClientRect.Bottom - FClientRect.Top - r.Top;
         Fill(0, 0, FClientRect.Right - FClientRect.Left, r.top);
         Fill(0, r.Bottom, FClientRect.Right - FClientRect.Left, FClientRect.Bottom - FClientRect.Top);
      end else begin
         r.Left := Round(((FClientRect.Right - FClientRect.Left) - (FClientRect.Bottom - FClientRect.Top)/k)/2);
         r.Right := FClientRect.Right - FClientRect.Left - r.Left;
         Fill(0, 0, r.left, FClientRect.Bottom - FClientRect.Top);
         Fill(r.Right, 0, FClientRect.Right - FClientRect.Left, FClientRect.Bottom - FClientRect.Top);      
      end;   
      SetStretchBltMode(DB.Canvas.Handle, HALFTONE);
      BackBmp.StretchDraw(DB.Canvas.Handle, r);
   end;
   DrawAnalogFace;
   DrawFatHand(HourHandPos(FDispTime), True);
   DrawFatHand(FDispTime.Minute, False);
   DrawSecondHand(FDispTime.Second);
   DB.Canvas.Pen.Color := DB.Canvas.Brush.Color;
   DB.Draw(Control.Canvas.Handle, 0, 0);   
end;

procedure THiClock.ClockOnPaint(Sender: PControl; DC: HDC) ;
begin
   PaintAnalogClock;
end;

function THiClock.GetTime;
var   ST:TSystemTime;
begin
   GetLocalTime(ST);
   with ST, Result do begin
      Hour := (wHour + FHourOffset) mod 24;
      Minute := wMinute;
      Second := wSecond;
   end;
end;

procedure THiClock.OnTimer;
begin
   FDispTime := GetTime;
   PaintAnalogClock;
end;

function WndProcClock(Ctl: PControl; var Msg: TMsg; var Rslt: Integer): Boolean;
var   fControl: THiClock;
begin
   fControl:= THiClock(Ctl.Tag);
   Result := False;
   case msg.message of
      WM_SIZE:begin
         GetClientRect(msg.hwnd, fControl.FclientRect);
         fControl.DB.Width := fControl.FClientRect.Right;
         fControl.DB.Height:= fControl.FClientRect.Bottom;
         InvalidateRect(msg.hwnd, nil, True);
      end;
   end;
end;

constructor THiClock.Create;
begin
   inherited;
   BackBmp := NewBitmap(0,0);
   ClockBmp := NewBitmap(0,0);
end;

procedure THiClock.Init;
var   Pos: Integer;
      vSize: Integer;
      hSize: Integer;
      DC: HDC;
begin
   Control := NewPanel(FParent, _prop_EdgeStyle);
   inherited;
   DC := GetDC(0);
TRY
   VRes  := GetDeviceCaps(DC, VERTRES);
   HRes  := GetDeviceCaps(DC, HORZRES);
   vSize := GetDeviceCaps(DC, VERTSIZE);
   hSize := GetDeviceCaps(DC, HORZSIZE);
FINALLY
   ReleaseDC(0, DC);
END;
   AspectV := (Longint(VRes) * MmPerDm) div Longint(vSize);
   AspectH := (Longint(HRes) * MmPerDm) div Longint(hSize);
   CircleTab := PPointArray(@ClockData);
   for Pos := 0 to HandPositions - 1 do CircleTab^[Pos].Y := Longint(CircleTab^[Pos].Y) * AspectV div AspectH;

   FTimer := NewTimer(1000);
   FTimer.OnTimer := OnTimer;
   FTimer.Enabled:=True;

   GetClientRect(Control.Handle, FClientRect);
   DB:= NewBitmap(Control.Width, Control.Height);
   FDispTime := GetTime;
   Control.Tag := LongInt(Self);
   Control.AttachProc(WndProcClock);
   Control.OnPaint := ClockOnPaint;
end;  

destructor THiClock.Destroy;
begin
   FTimer.free;
   DB.free;
   BackBmp.free;
   ClockBmp.free;
   inherited;
end;

end.